@@ -20,7 +20,7 @@
 from __future__ import absolute_import
 from __future__ import with_statement
 
-VERSION = (2, 7, 3, 28)
+VERSION = (2, 7, 3, 30)
 __version__ = ".".join(map(str, VERSION[0:4])) + "".join(VERSION[4:])
 __author__ = 'R Oudkerk / Python Software Foundation'
 __author_email__ = 'python-dev@python.org'
@@ -232,7 +232,11 @@ def JoinableQueue(maxsize=0):
     return JoinableQueue(maxsize)
 
 
-def Pool(processes=None, initializer=None, initargs=(), maxtasksperchild=None):
+def Pool(processes=None, initializer=None, initargs=(), maxtasksperchild=None,
+         timeout=None, soft_timeout=None, lost_worker_timeout=None,
+         max_restarts=None, max_restart_freq=1, on_process_up=None,
+         on_process_down=None, on_timeout_set=None, on_timeout_cancel=None,
+         threads=True, semaphore=None, putlocks=False, allow_restart=False):
     '''
     Returns a process pool object
     '''
