@@ -29,7 +29,7 @@ import warnings
 
 from . import Event, Process, cpu_count
 from . import util
-from .common import reset_signals, restart_state
+from .common import pickle_loads, reset_signals, restart_state
 from .compat import get_errno
 from .einfo import ExceptionInfo
 from .exceptions import (
@@ -233,28 +233,26 @@ def soft_timeout_sighandler(signum, frame):
 
 def worker(inqueue, outqueue, initializer=None, initargs=(),
            maxtasks=None, sentinel=None):
-    # Re-init logging system.
-    # Workaround for http://bugs.python.org/issue6721#msg140215
-    # Python logging module uses RLock() objects which are broken after
-    # fork. This can result in a deadlock (Issue #496).
-    logger_names = logging.Logger.manager.loggerDict.keys()
-    logger_names.append(None)  # for root logger
-    for name in logger_names:
-        for handler in logging.getLogger(name).handlers:
-            handler.createLock()
-    logging._lock = threading.RLock()
-
     pid = os.getpid()
     assert maxtasks is None or (type(maxtasks) == int and maxtasks > 0)
     put = outqueue.put
     get = inqueue.get
+    loads = pickle_loads
 
     if hasattr(inqueue, '_reader'):
 
-        def poll(timeout):
-            if inqueue._reader.poll(timeout):
-                return True, get()
-            return False, None
+        if hasattr(inqueue, 'get_payload'):
+            get_payload = inqueue.get_payload
+
+            def poll(timeout):
+                if inqueue._reader.poll(timeout):
+                    return True, loads(get_payload())
+                return False, None
+        else:
+            def poll(timeout):
+                if inqueue._reader.poll(timeout):
+                    return True, get()
+                return False, None
     else:
 
         def poll(timeout):  # noqa
@@ -1236,8 +1234,13 @@ class Pool(object):
             return result
 
     def terminate_job(self, pid, sig=None):
-        self.signalled.add(pid)
-        _kill(pid, sig or signal.SIGTERM)
+        try:
+            _kill(pid, sig or signal.SIGTERM)
+        except OSError, exc:
+            if get_errno(exc) != errno.ESRCH:
+                raise
+        else:
+            self.signalled.add(pid)
 
     def map_async(self, func, iterable, chunksize=None,
                   callback=None, error_callback=None):
